#include "ContextContactsAppUi.h"
#include "ContextContactsView.h"
#include "ContextContactsContainer.h"
#include "ContextContactsView2.h"
#include <ContextContacts.rsg>
#include "contextcontacts.hrh"

#include <avkon.hrh>
#include "icons.h"

#include "db.h"
#include <RPbkViewResourceFile.h>
#include <CPbkContactEditorDlg.h>
#include <CPbkContactEngine.h>
#include <CPbkSelectFieldDlg.h>
#include <CPbkContactItem.h>
#include <apgcli.h>
#include <eikmenup.h>
#include <CPbkViewState.h>
#include <bautils.h>
#include <aknmessagequerydialog.h> 
#include <flogger.h>
#include <EikDll.h>
#include "contextcontactsapp.h"

void CContextContactsAppUi::ConstructL()
{	
	CALLSTACKITEM(_L("CContextContactsAppUi::ConstructL"));

#ifndef __WINS__
	// works only for context_log on the emulator
	{ TRAPD(errs, StartStarterL(_L("contacts"), KUidContextContacts, true)); }
#endif

	BaseConstructL();

	// LOGGING ON or OFF
	TBool logging=ETrue;
	Settings().GetSettingL(18, logging);
	// FIXME
	iLog=Cfile_output_base::NewL(AppContext(), _L("book_log"), logging, false);

	if (!logging && iLog) iLog->paused=true;

	iJabberDb=CDb::NewL(AppContext(), _L("JABBER"), EFileWrite|EFileShareAny);
	iJabberData=CJabberData::NewL(AppContext(), *iJabberDb);

	iPresenceHolder=CPresenceHolder::NewL(*iJabberData);

	iBook=new (ELeave) phonebook(AppContext(), *iJabberData, *iPresenceHolder);
	iBook->ConstructL();

	iPhoneHelper = new (ELeave) phonehelper(*iBook);
	iPhoneHelper->ConstructL();

	if (iPresenceHolder) iPresenceHolder->AddListener(iBook);
	if (iPresenceHolder) iPresenceHolder->AddListener(this);

	iPresenceUpdater = new (ELeave) CPresenceUpdater(*iBook);
	iPresenceUpdater->ConstructL();

	// for phonebook dialogs
	iResource_files=new (ELeave) CArrayFixFlat<TInt>(5);
	TFileName resfile=_L("z:\\System\\data\\PBKVIEW.RSC");
	BaflUtils::NearestLanguageFile(iEikonEnv->FsSession(), resfile); //for localization
	iResource_files->AppendL(iEikonEnv->AddResourceFileL(resfile));

	CEikStatusPane* sp = StatusPane();
	iNaviPane = (CAknNavigationControlContainer*)sp->ControlL( 
        TUid::Uid(EEikStatusPaneUidNavi));
	iDecoratedTabGroup = iNaviPane->ResourceDecorator();
	if (iDecoratedTabGroup)
        {
		iTabGroup = (CAknTabGroup*) iDecoratedTabGroup->DecoratedControl();
        }
	iNaviPane->PushL(*iDecoratedTabGroup);

	iIconlist = new (ELeave) CAknIconArray(30);
	LoadIcons(iIconlist);

	iContactsView = new (ELeave) CContextContactsView(*iJabberData, iLog, iBook, iIconlist, *iPhoneHelper);
	CleanupStack::PushL( iContactsView );
	iContactsView->ConstructL();
	AddViewL( iContactsView );      // transfer ownership to CAknViewAppUi
	CleanupStack::Pop();    

	iGroupsView = new (ELeave) CContextContactsView2;
	CleanupStack::PushL( iGroupsView );
	iGroupsView->ConstructL();
	AddViewL( iGroupsView );      // transfer ownership to CAknViewAppUi
	CleanupStack::Pop();   

	iContactDetailView = new (ELeave) CContextContactsDetailView(*iPhoneHelper, iIconlist);
	CleanupStack::PushL( iContactDetailView );
	iContactDetailView->ConstructL();
	AddViewL( iContactDetailView );      // transfer ownership to CAknViewAppUi
	CleanupStack::Pop();   

	auto_ptr<CPresenceDetailView> detailview(CPresenceDetailView::NewL());
	AddViewL(detailview.get());
	iPresenceDetailView=detailview.release();

	auto_ptr<CSettingsView> iSettingsView(CSettingsView::NewL(AppContext(), *iBook));
	AddViewL(iSettingsView.get());
	iSettingsView.release();

	SetDefaultViewL(*iContactsView);
}

CContextContactsAppUi::~CContextContactsAppUi()
{
	CALLSTACKITEM(_L("CContextContactsAppUi::~CContextContactsAppUi"));

	iContactsView->before_exit();

	iEikonEnv->RemoveForegroundObserver(*this);
	for (int i=0; i<iResource_files->Count(); i++) {
		iEikonEnv->DeleteResourceFile((*iResource_files)[i]);
	}
	delete iResource_files;
	delete iPhoneHelper;

     	delete iLog;
	delete iBook;
	delete iPresenceHolder;
	delete iPresenceUpdater;
	delete iJabberData;
	delete iJabberDb;
	delete iDecoratedTabGroup; 
	delete iIconlist;
}

TKeyResponse CContextContactsAppUi::HandleKeyEventL(
    const TKeyEvent& aKeyEvent, TEventCode /*aType*/)
{
	CALLSTACKITEM(_L("CContextContactsAppUi::HandleKeyEventL"));

	if ( iTabGroup == NULL )
        {
		return EKeyWasNotConsumed;
        }
	TInt active = iTabGroup->ActiveTabIndex();
	TInt count = iTabGroup->TabCount();

	switch ( aKeyEvent.iCode )
        {
		case EKeyLeftArrow:
			if ( active > 0 )
			{
				active--;
				iTabGroup->SetActiveTabByIndex( active );
				ActivateLocalViewL(TUid::Uid(iTabGroup->TabIdFromIndex(active)));
			}
			break;
		case EKeyRightArrow:
			if( (active + 1) < count )
			{
				active++;
				iTabGroup->SetActiveTabByIndex( active );
				ActivateLocalViewL(TUid::Uid(iTabGroup->TabIdFromIndex(active)));
			}
			break;
		default:
			return EKeyWasNotConsumed;
			break;
        }
	return EKeyWasConsumed;
}

void CContextContactsAppUi::HandleCommandL(TInt aCommand)
{
	CALLSTACKITEM(_L("CContextContactsAppUi::HandleCommandL"));

	switch ( aCommand )
	{
		case EEikCmdExit:
		{
			//iContactsView->before_exit();
			Exit();
			break;
		}
		case EAknSoftkeyBack:
		{
			hide();
			break;
		}
		case EContextContactsCmdInfoMem:
		{
			TInt err, size, nb_contacts = -1;
			TInt64 free = -1;
                        TEntry entry;
			TVolumeInfo info;
			_LIT(KContactsDb, "c:\\System\\Data\\Contacts.cdb");
			
			nb_contacts = iBook->Count();
			err = Fs().Entry(KContactsDb, entry);
			if (err == KErrNone) size = entry.iSize;
			err = Fs().Volume(info);
			if (err == KErrNone) free = info.iFree;

			TReal x = size/1024;
			TReal y = (free/1024).GetTReal();

			HBufC * header = CEikonEnv::Static()->AllocReadResourceL(R_CONTACTS_INFO_BUF);
			CleanupStack::PushL(header);
			TBuf<200> message;
			TRealFormat realFormat = TRealFormat(30, 0);
			//FIX ME: use resource
			message.AppendFormat(_L("%d contacts\n"), nb_contacts);
			message.AppendNum(x, realFormat);
			message.Append(_L(" kB used\n"));
			message.AppendNum(y, realFormat);
			message.Append(_L(" kB free\n"));
			          			
			CAknMessageQueryDialog * dlg = CAknMessageQueryDialog::NewL(message);
			CleanupStack::PushL(dlg);
			dlg->PrepareLC(R_CONTACTS_INFO);
			dlg->QueryHeading()->SetTextL(*header);
			CleanupStack::Pop(dlg);
			
			dlg->RunLD();
			CleanupStack::PopAndDestroy(1); //header
			break;
		}
		
		case EContextContactsCmdOpen:
		{
			CPbkViewState* pbkViewParam = CPbkViewState::NewLC(); 
			TInt contact_id = iContactsView->GetCurrentContactId(); 
			pbkViewParam->SetFocusedContactId( contact_id ); 
			HBufC8* paramBuf = pbkViewParam->PackLC();
			ActivateLocalViewL(KDetailViewId, CPbkViewState::Uid(), *paramBuf); 
			CleanupStack::PopAndDestroy(2); // paramBuf, pbkViewPara
			break;
		}
		case EContextContactsCmdSimDir:
		{
			TFileName appName=_L("z:\\System\\Apps\\SimDirectory\\SimDirectory.app");
			const TUid KSimDirUid = { 0x101F4CF6 }; 

			// FIXME : this calling of simdirectory doesn't work!
                        // CCoeAppUi::ActivateViewL(TVwsViewId(KSimDirUid, TUid::Uid(1)));
			CApaCommandLine* cmdLine=CApaCommandLine::NewLC();
			cmdLine->SetLibraryNameL(appName);
			//cmdLine->SetDocumentNameL(docName);
			cmdLine->SetCommandL(EApaCommandRunWithoutViews);
			RApaLsSession ls;
			User::LeaveIfError(ls.Connect());
			CleanupClosePushL(ls);
			User::LeaveIfError(ls.StartApp(*cmdLine));
			CleanupStack::PopAndDestroy(2); // ls and cmdLine
			break;
		}

		case EContextContactsCmdSettings:
		{
			//ActivateLocalViewL(KSettingsViewId);
			break;
		}
		default:
			break;      
        }
}

void CContextContactsAppUi::hide()
{
	CALLSTACKITEM(_L("CContextContactsAppUi::hide"));


	RWsSession& wsSession=CEikonEnv::Static()->WsSession();
	
	TApaTask task(wsSession);
	task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
	task.SendToBackground();
}

void CContextContactsAppUi::HandleGainingForeground()
{
	CALLSTACKITEM(_L("CContextContactsAppUi::HandleGainingForeground"));


	if (iLog) {
		iLog->write_time();
		iLog->write_to_output(_L("To foreground"));
		iLog->write_nl();
	}
	if (iPresenceUpdater) iPresenceUpdater->Start();
}

void CContextContactsAppUi::HandleLosingForeground()
{
	CALLSTACKITEM(_L("CContextContactsAppUi::HandleLosingForeground"));


	if (iLog) {
		iLog->write_time();
		iLog->write_to_output(_L("To background\n"));
		iLog->write_nl();
	}
	if (iPresenceUpdater) iPresenceUpdater->Stop();

}

void CContextContactsAppUi::Notify(const TDesC& /*aMessage*/)
{
	CALLSTACKITEM(_L("CContextContactsAppUi::Notify"));

	// no impl	
}

void CContextContactsAppUi::PresenceChangedL(TInt /*ContactId*/, MPresenceData &/*info*/)
{
	CALLSTACKITEM(_L("CContextContactsAppUi::PresenceChangedL"));

	// no implementation
}

void CContextContactsAppUi::DisplayPresenceDetailsL(const TDesC& Name, const MPresenceData* PresenceData)
{
	CALLSTACKITEM(_L("CContextContactsAppUi::DisplayPresenceDetailsL"));


	if (PresenceData == 0) return;
	iPresenceDetailView->SetData(Name, PresenceData);
	ActivateLocalViewL(KPresenceDetailView);
}

